%    <CustomTools>
%      <Menu>
%        <Item name="autoSegmentChannel" icon="Matlab">
%          <Command>MatlabXT::autoSegmentChannel(%i)</Command>
%        </Item>
%      </Menu>
%    </CustomTools>







function [] = autoSegmentChannel(aImarisApplicationID)
%UNTITLED3 Summary of this function goes here
%   Detailed explanation goes here
   
ProgressDisplay = waitbar(0, 'Getting Image,please wait...');
if ~isnumeric(aImarisApplicationID)
    aImarisApplicationID=str2num(aImarisApplicationID);
end

if ~isnumeric(aImarisApplicationID) && ~ischar(aImarisApplicationID)
    aImarisApplicationID=0;
end
global vImarisApplication;

if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
   vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end

vDataSet = vImarisApplication.GetDataSet.Clone;
vSurpassScene = vImarisApplication.GetSurpassScene;
aType = Imaris.tType.eTypeFloat;

if strcmp(vDataSet.GetType,'eTypeUInt8')
    vImarisApplication.GetDataSet.SetType(aType);
elseif strcmp(vDataSet.GetType,'eTypeUInt16')
    vImarisApplication.GetDataSet.SetType(aType);
end    



   

%Get DataSet Info for voxel sizing etc...
vDataMin = [vDataSet.GetExtendMinX, vDataSet.GetExtendMinY, vDataSet.GetExtendMinZ];
vDataMax = [vDataSet.GetExtendMaxX, vDataSet.GetExtendMaxY, vDataSet.GetExtendMaxZ];
vDataSize = [vDataSet.GetSizeX, vDataSet.GetSizeY, vDataSet.GetSizeZ];

vTimeMax = vDataSet.GetSizeT - 1;

SizeX = vDataSet.GetSizeX;
SizeY = vDataSet.GetSizeY;
SizeZ = vDataSet.GetSizeZ;
PixelSize = (vDataSet.GetExtendMaxX)/SizeX;
PixelDepth = (vDataSet.GetExtendMaxZ)/SizeZ;
vNumberOfChannels = vDataSet.GetSizeC;

for i=1:vNumberOfChannels
    ChNames{i}=char(vDataSet.GetChannelName(i-1));
end

if vNumberOfChannels==1
    choice=1;
else   
    waitbar(0, ProgressDisplay,'Please choose channel from popup');
    choice = menu('choose Channel to auto segment',ChNames);
end
if isequal(ChNames{choice},'(name not specified)')
    ChNames{choice}=['Channel' num2str(choice)];
end    
outputChannelName=['auto Segment of ' ChNames{choice}];


%add Imaris Channel
vDataSet.SetSizeC(vNumberOfChannels + 1);
vDataSet.SetChannelName(vNumberOfChannels,outputChannelName);
vDataSet.SetChannelColorRGBA(vNumberOfChannels, 16711935);


%get user selected image segment and return each time point

for vTime=1:vTimeMax+1
    waitbar((vTime)/(vTimeMax+1),ProgressDisplay, 'Getting time point');
    [Image,ch1voxelsize] = XTGetImarisImageMWA(vImarisApplication,vTime,choice);
    waitbar((vTime)/(vTimeMax+1),ProgressDisplay, 'calculating');
    if SizeZ>1
        [threshVal{vTime},aaa,cVals{vTime}] = getHistThreshActual_MWA(max(Image,[],3),0);
    else
        [threshVal{vTime},aaa,cVals{vTime}] = getHistThreshActual_MWA(Image,0);
    end
       
    nVals=max(size(threshVal{vTime}));
    if nVals>1
        display('finding best of multiple threshold results');
        for i=1:nVals
            BW{i}=Image>=threshVal{vTime}(i);
            bwStats=regionprops(bwlabeln(BW{i}),'Area');
            [nRois,dontcare]=size(bwStats);  
            AreaMatrix=[bwStats.Area];
            sumArea=sum(AreaMatrix);
            stdArea=std(AreaMatrix);
            thisMetric(vTime,i) = ((nRois)^2)/(sumArea*cVals{vTime}(i));
        end
        indexOfGoodThresh{vTime} = find(thisMetric(vTime,1:end)==min(thisMetric(vTime,1:end)));   
        if max(size(indexOfGoodThresh{vTime}))>1 && vTime>1
            thisSize = max(size(thisMetric(vTime)));
            oldSize = max(size(thisMetric(vTime-1)));
            if oldSize==thisSize
                indexOfGoodThresh{vTime}=indexOfGoodThresh{vTime-1};
            end    
        end 
        timeThresh{vTime}=threshVal{vTime}(indexOfGoodThresh{vTime});
        Image=double(BW{indexOfGoodThresh{vTime}}).*double(Image);
        waitbar((vTime)/(vTimeMax+1),ProgressDisplay, 'updating time point');
        XTSetImarisImageMWA(vImarisApplication, vDataSet, Image, vTime, vNumberOfChannels+1);
    else
        timeThresh{vTime}=threshVal{vTime};
        Image=double(Image>threshVal{vTime}).*double(Image);
        waitbar((vTime)/(vTimeMax+1),ProgressDisplay, 'updating time point');
        XTSetImarisImageMWA(vImarisApplication, vDataSet, Image, vTime, vNumberOfChannels+1);
    end
    
end    
vImarisApplication.SetDataSet(vDataSet);
close(ProgressDisplay);   

stopHere2DeBug=3;


end    






function [threshVal,CorrCurve,corrVal]=getHistThreshActual_MWA(InCh,Exact)
%InCh is a 2D Image
%Exact is a binary value.  1 for exact-slower. 0 for rough estimate



iDx=[];
threshVal=[];
CorrCurve=[];

if max(size(size(InCh)))>2 || max(size(size(InCh)))==1
    display('this function only supports 2D inputs');
    return;
end
bit8Max=intmax('uint8');
bit16Max=intmax('uint16');

maxVal=double(max(max(InCh)));
if maxVal<=bit8Max
    [counts,X]=MYimhist2D(uint8(InCh),maxVal);
    BW=ones(maxVal,1);
    maxVal2=maxVal;
end

if maxVal>bit8Max && maxVal<=bit16Max
    [counts,X]=MYimhist2D(uint16(InCh),1024);
    BW=ones(1024,1);
    maxVal2=1024;
end 

if maxVal>bit16Max
    [counts,X]=MYimhist2D(uint32(InCh),2048);
    BW=ones(2048,1);
    maxVal2=2048;
end


for i=1:maxVal2
    BW(i)=0;
    CorrCurve(i)=histCorr_MWA(counts,X,BW);   
end
[peakVal,peakLoc]=findpeaks(CorrCurve);
if max(size(peakLoc))>2
    display('low signal to noise');
    testpeakLocs = peakLoc(find(peakVal==max(peakVal)));
    testpeakVals = peakVal(find(peakVal==max(peakVal)));
    if max(size(testpeakLocs))>1
        peakLocs(1)=max(testpeakLocs);
        peakVals(1)=CorrCurve(peakLocs(1));
    else
        peakLocs(1)=testpeakLocs;
        peakVals(1)=testpeakVals;
    end    
    peakVal(find(peakLoc==peakLocs(1)))=0;
    testpeakLocs = peakLoc(find(peakVal==max(peakVal)));
    testpeakVals = peakVal(find(peakVal==max(peakVal)));
    if max(size(testpeakLocs))>1
        peakLocs(2)=max(testpeakLocs);
        peakVals(2)=CorrCurve(peakLocs(1));
    else
        peakLocs(2)=testpeakLocs;
        peakVals(2)=testpeakVals;
    end    
end    
if max(size(peakLoc))<3 && ~isempty(peakLoc)
    peakLocs=peakLoc;
    peakVals=peakVal;
end    
if isempty(peakLoc)
    peakLocs=find(CorrCurve==max(CorrCurve));
    peakVals=CorrCurve(peakLoc);
    if max(size(peakLocs))>1
        peakLocs=max(peakLocs);
        peakVals=CorrCurve(peakLocs);
    end    
end    

indexOFx=peakLocs;
threshVal=X(indexOFx);
corrVal=peakVals;



stopHere2DeBug=3;



end    



function [CorrVal]=histCorr_MWA(counts,X,BW)

%check for same size
sizeCounts=size(counts);
sizeBW=size(BW);

if ~isequal(sizeCounts,sizeBW)
    display('inputs A and B [[histCorr_MWA(A,B)]] must be the same size and dimension');
    CorrVal=NaN;
    return;
end

%check dims of counts
if max(size(size(counts)))>2 || min(size(counts))>1
    display('this should be a 1D histogram input, counts is apparently of higher dims');
    CorrVal=NaN;
    return;
end

%check dims of BW
if max(size(size(BW)))>2 || min(size(BW))>1
    display('this should be a 1D Binary input, BW is apparently of higher dims');
    CorrVal=NaN;
    return;
end



%if the previous checks pass, then counts and BW should be lineraly indexed
cSum=0;
bSum=0;
nPix=0;
for ii=2:max(size(X))
    cSum=( X(ii) * counts(ii)) + cSum;
    bSum=(BW(ii) * counts(ii)) + bSum;
    nPix=counts(ii)+nPix;
end
cMean=cSum/nPix;
bMean=bSum/nPix;


covSum=0;
sigCsum=0;
sigBsum=0;
Tcounts=0;
for j=1:max(size(X))
    covSum = covSum +((X(j)-cMean)*(BW(j)-bMean)*(counts(j)));
    sigCsum = sigCsum + (counts(j)*(X(j)-cMean)^2);
    sigBsum = sigBsum + (counts(j)*(BW(j)-bMean)^2);
end

CorrVal = covSum/(sqrt(sigCsum*sigBsum));


end






%
%
%  Send Image to Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Description:
%   
%   Change the Imaris DataSet.
%   This function can be used as utility for other functions; It is not
%       called directly by Imaris.
%
%

function XTSetImarisImageMWA(aImarisApplication, vDataSet, aImage, aTimeIndex, aChannelIndex)


if vDataSet.GetSizeX ~= size(aImage,1)  || vDataSet.GetSizeY ~= size(aImage,2) || vDataSet.GetSizeZ ~= size(aImage,3)
   vDataSet.Resize(0,size(aImage,1), 0,size(aImage,2), 0, size(aImage,3), 0, vDataSet.GetSizeC, 0, vDataSet.GetSizeT);
end
if size(aImage,1) == vDataSet.GetSizeX && ...
        size(aImage,2) == vDataSet.GetSizeY && ...
        size(aImage,3) == vDataSet.GetSizeZ
   if strcmp(vDataSet.GetType,'eTypeUInt8')
        vMin = min(reshape(aImage,[1,numel(aImage)]));
        if vMin < 0
            aImage = aImage + vMin;
        end
        vMax = max(reshape(aImage,[1,numel(aImage)]));
        if vMax > (2^8-1)
            aImage = aImage*(2^8-1)/vMax;
        end
        vDataSet.SetDataVolumeBytes(uint8(aImage), aChannelIndex-1, aTimeIndex-1);
   elseif strcmp(vDataSet.GetType,'eTypeUInt16')
        vMin = min(reshape(aImage,[1,numel(aImage)]));
        if vMin < 0
            aImage = aImage + vMin;
        end
        vMax = max(reshape(aImage,[1,numel(aImage)]));
        if vMax > (2^16-1)
            aImage = aImage*(2^16-1)/vMax;
        end 
        vDataSet.SetDataVolumeShorts(uint16(aImage), aChannelIndex-1, aTimeIndex-1);
   elseif strcmp(vDataSet.GetType,'eTypeFloat')
        vDataSet.SetDataVolumeFloats(single(aImage), aChannelIndex-1, aTimeIndex-1);
   end
end
% aImarisApplication.SetDataSet(vDataSet);
end




%
%
%  Get Image from Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Description:
%   
%   Get and returns the Imaris DataSet.
%   This function can be used as utility for other functions; It is not
%       called directly by Imaris.
%
%

function [aImage, aVoxelSize] = XTGetImarisImageMWA(aImarisApplication, aTimeIndex, aChannelIndex)

vImarisDataSet = aImarisApplication.GetDataSet;

vExtendMin = [vImarisDataSet.GetExtendMinX, vImarisDataSet.GetExtendMinY, ...
  vImarisDataSet.GetExtendMinZ];
vExtendMax = [vImarisDataSet.GetExtendMaxX, vImarisDataSet.GetExtendMaxY, ...
  vImarisDataSet.GetExtendMaxZ];
vDataSize = [vImarisDataSet.GetSizeX, vImarisDataSet.GetSizeY, ...
  vImarisDataSet.GetSizeZ, vImarisDataSet.GetSizeC, vImarisDataSet.GetSizeT];

aVoxelSize = (vExtendMax - vExtendMin) ./ vDataSize(1:3);

vMinT = 0;
vMinC = 0;

if nargin == 3 && aChannelIndex<=vDataSize(4) && aTimeIndex<=vDataSize(5)

  vMinT = aTimeIndex-1;
  vDataSize(5) = 1;
  vMinC = aChannelIndex-1;
  vDataSize(4) = 1;
  
elseif nargin == 2 && aTimeIndex<=vDataSize(5)

  vMinT = aTimeIndex-1;
  vDataSize(5) = 1;

end

vSize = [1, prod(vDataSize(1:3)), vDataSize(4:5)];

if strcmp(vImarisDataSet.GetType,'eTypeUInt8')
  aImage = zeros(vSize, 'uint8');
elseif strcmp(vImarisDataSet.GetType,'eTypeUInt16')
  aImage = zeros(vSize, 'uint16');
else
  aImage = zeros(vSize, 'single');
end

for vTimeIndex = 1:vDataSize(5)
  for vChannelIndex = 1:vDataSize(4)
    if strcmp(vImarisDataSet.GetType,'eTypeUInt8')
      aImage(1, :, vChannelIndex, vTimeIndex) = typecast(...
        vImarisDataSet.GetDataVolumeAs1DArrayBytes( ...
        vMinC + vChannelIndex-1, vMinT + vTimeIndex-1), 'uint8');
    elseif strcmp(vImarisDataSet.GetType,'eTypeUInt16')
      aImage(1, :, vChannelIndex, vTimeIndex) = typecast(...
        vImarisDataSet.GetDataVolumeAs1DArrayShorts( ...
        vMinC + vChannelIndex-1, vMinT + vTimeIndex-1), 'uint16');
    else
      aImage(1, :, vChannelIndex, vTimeIndex) = ...
        vImarisDataSet.GetDataVolumeAs1DArrayFloats( ...
        vMinC + vChannelIndex-1, vMinT + vTimeIndex-1);
    end
  end
end
aImage = reshape(aImage, vDataSize);
end

function [ counts,X ] = MYimhist2D( I,nBins )
%UNTITLED3 Summary of this function goes here
%   Detailed explanation goes here

if ~isequal(ndims(I),2)
    error('input Image should be 2 dimensional');
end

maxVal=double(max(max(I)));
X=ones(1,nBins);
for i=1:nBins %get X bin values
    X(i)=(maxVal/nBins)*i;
end

newI=reshape(I,[numel(I),1]);
    
counts=histc(newI,[0:nBins-1]);



end


